/*-
 * Copyright (c) 1993-1994 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and the Network Research Group at
 *      Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
static const char rcsid[] =
    "@(#) $Header: /usr/src/mash/repository/mash/mash-1/codec/decoder-mme.cc,v 1.1.1.1 1996/12/07 04:27:56 mccanne Exp $ (LBL)";

#include <osfcn.h>
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
extern "C" {
/*XXX*/
#undef VOID
#include <mme/mme_api.h>
}
#include "inet.h"
#include "packet.h"
#include "video.h"
#include "decoder-mme.h"
#include "color.h"
#include "renderer.h"
#include "idlecallback.h"

extern "C" {
#include <sys/param.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#ifdef __osf__
int shmget(key_t, u_int, u_int);
#endif
#ifdef mips
#include <mips/cachectl.h>
int cacheflush(char* addr, int len, int cache);
#endif
}

class MmeRenderer : public Renderer, public IdleCallback {
public:
	MmeRenderer(VideoWindow* w, int inw, int inh, int inq, int pri);
	~MmeRenderer();
	MmeRenderer* next_;

	void recv(u_char* frame, int len);
	int match(VideoWindow*, int inq, int inw, int inh);
	void reset(int inq, int inw, int inh);
	void sleep();
	inline int active() { return (active_); }
	struct ibm {
		EXBMINFOHEADER b;
		JPEGINFOHEADER jpeg;
		RGBQUAD rgb[256];
	};
	struct obm {
		BITMAPINFOHEADER b;
		RGBQUAD rgb[256];
	};
protected:
	virtual void idle_callback();
	void icopen();
	virtual void timeout();

	void render();
	void decompress();

	void initcolors();

	int inq_;		/* input quantization */

	HIC handle_;

	SharedVideoImage* vbd_;
	SharedVideoImage* vbx_;

	void allocimage();

	int active_;
	ibm* in_;
	obm* out_;
	u_char* inb_;		/* dma buffer for input (compressed) frame */

	u_char map_[256];
};

MmeRenderer::MmeRenderer(VideoWindow* window, int inw, int inh, int inq, int pri)
	: Renderer(window, pri, inw, inh),
	  inq_(inq), next_(0), active_(1)
{
	init(window, pri);
	handle_ = 0;
	icopen();
	allocimage();

	/*XXX don't starve lib*/
	while (mmeCheckForCallbacks())
		mmeProcessCallbacks();
}

MmeRenderer::~MmeRenderer()
{
	if (handle_ != 0)
		ICClose(handle_);
}

void MmeRenderer::icopen()
{
	handle_ = ICOpen(ICTYPE_VIDEO, 0, ICMODE_DECOMPRESS);
	if (handle_ == 0) {
		/*XXX*/
		fprintf(stderr, "vic: mme ICOpen failed\n");
		exit(1);
	}
	in_ = (ibm*)mmeAllocMem(sizeof(*in_));
	memset(in_, 0, sizeof(*in_));

	in_->b.bmi.biClrUsed = 256;
	in_->b.bmi.biSize = sizeof(*in_) - sizeof(in_->rgb);/*sigh*/
	in_->b.bmi.biWidth = 320;/*XXX*/
	in_->b.bmi.biHeight = 240;/*XXX*/
	in_->b.bmi.biPlanes = 1;
	in_->b.bmi.biBitCount = 24;
	in_->b.bmi.biCompression = MJPG_DIB; /*XXX*/
	in_->b.biExtDataOffset = sizeof(in_->b);
	in_->jpeg.JPEGSize = sizeof(in_->jpeg);
	in_->jpeg.JPEGProcess = JPEG_PROCESS_BASELINE;
	in_->jpeg.JPEGColorSpaceID = JPEG_YCbCr;
	in_->jpeg.JPEGBitsPerSample = 8;
	in_->jpeg.JPEGHSubSampling = 2;/*XXX*/
	in_->jpeg.JPEGVSubSampling = 1;/*XXX*/

	out_ = (obm*)mmeAllocMem(sizeof(*out_));
	memset(out_, 0, sizeof(*out_));
	out_->b.biSize = sizeof(*out_) - sizeof(out_->rgb);/*sigh*/
	out_->b.biWidth = outw_;
	out_->b.biHeight = outh_;
	out_->b.biPlanes = 1;/*XXX?*/
	out_->b.biBitCount = 8;
	int size = outw_ * outh_;
	out_->b.biSizeImage = outw_ * outh_;
	out_->b.biCompression = BICOMP_DECXIMAGEDIB;
	out_->b.biClrUsed = 128;

	MMRESULT s;
#ifdef notdef
	s = ICGetFormatSize(handle_, in_, out_);
#endif
	initcolors();

	/*XXX 30k? */
#define MAX_JPEG_FRAME (30*1024)
	inb_ = (u_char*)mmeAllocBuffer(MAX_JPEG_FRAME);
	if (inb_ == 0) {
		fprintf(stderr, "vic: couldn't allocate mme frame memory\n");
		/*XXX*/
		exit(1);
	}
	s = ICDecompressBegin(handle_, in_, out_);
	if (s != ICERR_OK) {
		fprintf(stderr, "vic: ICDecompressBegin failed (%d)\n", s);
		/*XXX*/
		exit(1);
	}
}

void MmeRenderer::timeout()
{
	active_ = 1;
	msched(update_);
}

void MmeRenderer::sleep()
{
	windows_ = 0;
}

inline int MmeRenderer::match(VideoWindow* w, int inq, int inw, int inh)
{
	return (inq == inq_ && inw == inw_ && inh == inh_ &&
		Renderer::match(w));
}

void MmeRenderer::initcolors()
{
	const ColorModel& cm = ColorModel::instance();

	/* XXX the mme library does not seem to be looking at this */
	in_->b.bmi.biClrUsed = 0;

	out_->b.biClrUsed = 128;
	out_->b.biClrImportant = 1;
	MMRESULT s;
	s = ICDecompressGetPalette(handle_, in_, out_);
	if (s < 0)
		fprintf(stderr,
			"vic: ICDecompressGetPalette failed (%d)\n", s);
	printf("out colors %d clrImportant %d\n", out_->b.biClrUsed,
	       out_->b.biClrImportant);

	for (int i = 0; i < 128; ++i) {
		int index = cm.rgb_closest(out_->rgb[i].rgbRed,
					   out_->rgb[i].rgbGreen,
					   out_->rgb[i].rgbBlue);
		map_[i] = index;
	}

#ifdef notyet
	int mono = 0;
	int ncolors = 128;
#ifdef	notdef
	struct mme_color colors[256];
	dc_ = jvs_alloc_dctx(jvs_, inq_, inw_, inh_, &rw_, &rh_, &ncolors,
			       mono, 8, 0, 0, alignment, colors);
	if (dc_ < 0)
		jvs_error(jvs_, "jvs_alloc_dctx");

#ifdef notdef
	/*XXX*/
	extern int useJvColors;
	if (useJvColors)
		Ditherer::instance()->jvmap(nvmap);
#endif
	if (nvcolors != ncolors) {
		nvcolors = ncolors;
		const ColorCube& cube = ColorCube::instance();
		mme_color* p = colors;
		for (int i = 0; i < ncolors; ++p, ++i) {
			int index = cube.closest(p->r, p->g, p->b);
			nvmap[i] = index;
		}
	}
	if (jvs_setcolormap(jvs_, dc_, nvmap, ncolors) < 0)
		jvs_error(jvs_, "jvs_setcolormap");
#endif
#endif
}

void MmeRenderer::reset(int inq, int inw, int inh)
{
	/*XXX*/
	printf("vic: MmeRenderer reset yikes!\n");

	/*
	 * Make sure all outstanding putimage's finish.
	 * Syncing with the first window, syncs the server,
	 * and hence all of them.
	 */
	if (windows_)
		windows_->sync();
	
	inq_ = inq;
	inw_ = inw;
	inh_ = inh;

	int w = width_;
	int h = height_;
#ifdef	notdef
	if (jvs_resize(jvs_, dc_, inq_, inw_, inh_, &w, &h, alignment) < 0)
		jvs_error(jvs_, "jvs_resize");
#endif
#ifdef notdef
	if (w != rw_ || h != rh_) {
		rw_ = w;
		rh_ = h;
		allocimage();
	}
	if (qh_ != 0)
		decompress();
#endif
}

/*XXX maybe it's easiest to just nuke reset*/
void MmeRenderer::allocimage()
{
#ifdef notyet
	/*XXX*/
	if (vb0_.image != 0) {
		(void)mmeFreeBuffer(vbd_->buffer());
		(void)mmeFreeBuffer(vbx_->buffer());
		delete vb0_.image;
		delete vb1_.image;
	}
#endif
	Tk_Window tk = windows_->tkmain();

	/*XXX +12 for NTSC crap */
	imagesize_ = outw_ * outh_;
	mmeShmSegmentInfo si;
	u_char* p = (u_char*)mmeAllocBufferAndGetShminfo(imagesize_, &si);
	if (p != (u_char*)si.shmaddr)
		/*XXX*/
		fprintf(stderr, "vic: mmeAllocBufferAndGetShminfo problem!\n");

	vbd_ = new SharedVideoImage(tk, outw_, outh_, p, si.shmid);
	p = (u_char*)mmeAllocBufferAndGetShminfo(imagesize_, &si);
	vbx_ = new SharedVideoImage(tk, outw_, outh_, p, si.shmid);
}

void MmeRenderer::render()
{
	/*
	 * If any of the windows haven't been fully updated yet,
	 * we must be running behind.  Just drop this frame.
	 */	 
	for (VideoWindow* p = windows_; p != 0; p = p->next_)
		if (p->copying())
			return;

	MMRESULT s;
	s = ICDecompress(handle_, 0, (BITMAPINFOHEADER*)in_, inb_,
			 (BITMAPINFOHEADER*)out_, vbd_->buffer());
	if (s != ICERR_OK) {
		fprintf(stderr, "vic: ICDecompress failed (%d)\n", s);
		return;
	}

	/*XXX this is BOGUS */
	u_char* cp = (u_char*)vbd_->buffer();
	for (int i = 0; i < imagesize_; ++i) {
		int v = cp[i];
		if (v >= 128)
			printf("bad pixel\n");
		else
			cp[i] = map_[v];
	}

	
	/*
	 * Swap the decompression and displayed
	 * images, and fire off the XShmPutImage.
	 */
	SharedVideoImage* t = vbx_;
	vbx_ = vbd_;
	vbd_ = t;

	for (p = windows_; p != 0; p = p->next_)
		p->render(vbx_);

	if (pri_ == PRI_THUMBNAIL)
		active_ = 0;

	/*XXX don't starve lib*/
	while (mmeCheckForCallbacks())
		mmeProcessCallbacks();
}

void MmeRenderer::idle_callback()
{
	render();
}

void MmeRenderer::recv(u_char* frame, int cc)
{
	/*
	 * Copy packet data into a dma buffer; XXX would be nice
	 * to eliminate this copy.
	 */
	bcopy(frame, inb_, cc);
	/*XXX it might be nice if the fucking documentation said
	  you needed to stuff this length hear.  Instead this info
	  is buried in some header file. */
	in_->b.bmi.biSizeImage = cc;
	/*
	 * Don't render right now, because we might be getting called
	 * back by local loopback (and hence invoking the mme library
	 * from a callback, which is illegal).  XXX Maybe the local
	 * loopback function should be deferred.
	 */
	idle_sched();
}

MmeRenderer* MmeDecoder::free_;
int MmeDecoder::nfree_;

MmeDecoder::MmeDecoder(Tcl& tcl, Tk_Window tk, const u_char* pkthdr)
	: Decoder(tcl, tk, sizeof(jpeghdr)), engines_(0)
{
	jpeghdr* p = (jpeghdr*)pkthdr;
	inq_ = ntohs(p->quantization);
	inw_ = ntohs(p->width);
	inh_ = ntohs(p->height);
}

MmeDecoder::~MmeDecoder()
{
	/* XXX spin through renderers and delete them */
	MmeRenderer* p = engines_;
	while (p != 0) {
		MmeRenderer* n = p->next_;
		release(p);
		p = n;
	}
}

void MmeDecoder::release(MmeRenderer* p)
{
	p->sleep();
	p->next_ = free_;
	free_ = p;
	++nfree_;
	/* XXX throw away if nfree to big */
}

/*XXX BUSTED*/
int MmeDecoder::recv(rtphdr* rh, u_char* vh, u_char* bp, int cc, int)
{	
	jpeghdr* vp = (jpeghdr*)vh;
	if (inq_ != ntohs(vp->quantization) ||
	    inw_ != ntohs(vp->width) ||
	    inh_ != ntohs(vp->height)) {
		/*
		 * The JPEG parameters have changed.
		 * It's easy enough to reprogram the renderers,
		 * so just do that.
		 */
		inq_ = ntohs(vp->quantization);
		inw_ = ntohs(vp->width);
		inh_ = ntohs(vp->height);
		for (MmeRenderer* p = engines_; p != 0; p = p->next_)
			p->reset(inq_, inw_, inh_);
	}
	for (MmeRenderer* p = engines_; p != 0; p = p->next_)
		if (p->active())
			p->recv(bp, cc);
}

MmeRenderer* MmeDecoder::allocate(VideoWindow* w, int pri)
{
	MmeRenderer* p;
	for (MmeRenderer** pp = &free_; (p = *pp) != 0; pp = &p->next_) {
		if (p->match(w, inq_, inw_, inh_)) {
			*pp = (*pp)->next_;
			p->init(w, pri);
			return (p);
		}
	}
	for (pp = &free_; (p = *pp) != 0; pp = &p->next_) {
		if (p->Renderer::match(w)) {
			*pp = (*pp)->next_;
			p->init(w, pri);
			p->reset(inq_, inw_, inh_);
			return (p);
		}
	}
	p = free_;
	if (p != 0) {
		free_ = p->next_;
		p->init(w, pri);
		p->reset(inq_, inw_, inh_);
		return (p);
	}
	return (new MmeRenderer(w, inw_, inh_, inq_, pri));
}

void MmeDecoder::attach(VideoWindow* w, int pri)
{
	for (MmeRenderer* p = engines_; p != 0; p = p->next_) {
		if (p->Renderer::match(w)) {
			p->attach(w);
			return;
		}
	}
	p = allocate(w, pri);
	p->next_ = engines_;
	engines_ = p;
}

void MmeDecoder::detach(VideoWindow* w)
{
	MmeRenderer** np = &engines_;
	MmeRenderer* p;
	for (;;) {
		p = *np;
		/*XXX*/
		if (p == 0)
			return;
		if (p->contains(w))
			break;
		np = &p->next_;
	}
	p->detach(w);
	/*
	 * Clear out the image so that the Video class does
	 * not try to use it before another window gets attached
	 * and replaces the image.
	 */
	w->setimage(0);
	if (p->empty()) {
		*np = p->next_;
		release(p);
	}
}
