/*
 * Copyright (c) 1996-7 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by the Computer Systems
 *       Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * ---------------------------
 *
 * Filename:  array.H	-- Teck-Lee Tung Fri Dec 27 1996
 *
 * Description: Dynamic array 
 *
 * $Header: /usr/src/mash/repository/mash/mash-1/mbv2/array.h,v 1.2 1998/12/08 23:42:20 yatin Exp $
 */

#ifndef ARRAY_H
#define ARRAY_H

#include "misc/all-types.h"


//
// generic dynamic array of T items 
//     grow if accessed pass the limit
//     all unassigned elements are set to NULL
// Note: all elemen
//
template<class T>
class Array {
public:
	Array(int stepSize) : step_size_(stepSize) {
		pT_=new T[step_size_];
		memset((char*)pT_, 0, sizeof(T)*step_size_ ); 
		array_size_ = step_size_;
	}
	~Array() { 
		delete[] pT_; 
	}

	T& operator[](u_int32_t index) {
		if (index >= array_size_) 
			grow(index);
		return pT_[index];
	}

	u_int32_t get_size() { return array_size_; }

protected:
	void grow(u_int32_t index) {
		while (array_size_ + step_size_ <= index) {
			step_size_ <<= 1;
		}        
		T* pNewT = new T[step_size_ + array_size_];
		memcpy(pNewT, pT_, array_size_*sizeof(T));
		memset((char*)(pNewT+array_size_), 
		       0, (step_size_)*sizeof(T));
		delete[] pT_ ;
		pT_ = pNewT;
		array_size_ += step_size_;
		if (step_size_ <= array_size_) {            
			step_size_ <<= 1; // double the increase for next time
		}
	}    

private:                
	u_int32_t step_size_;
	u_int32_t array_size_;
	T *pT_;    
};


#endif /* #ifdef ARRAY_H */ 
