import Application NewscastUI WebPoller/NYTIndex \
	AnnounceListenManager/AS/Client/CASA

#
# Newscaster is a information dissemination application using SRMv2
# Author: Tina Wong
# Creation Date: 4/20/99
# Status: pre-alpha
#
Class NewscastApp -superclass Application -configuration {
	cacheDir ~/.mash/srmcache/
}

#
# Constructor
#
NewscastApp public init { argv } {
	$self next newscast

	# parse arguments
	set o [$self options]
	$o register_boolean_option -recv recv
	$o register_boolean_option -src src
	set netspec [$o parse_args $argv]
	global ssd
	if { [$self get_option recv] != "" } {	
		set ssd(ctype) "recv"
	} elseif { [$self get_option src] != "" } {
		set ssd(ctype) "src"		
	} else {
		puts stderr "newscast: must specify as recv or src"
		$self fatal
	}
	
	if { $netspec == "" } {
		puts stderr "newscast: [-recv|-src] address"
		$self fatal "unknown/invalid address"
	}

	# cache and index db
	$self instvar cache_dir_ index_filename_
	$self create_dir [$self get_option cacheDir]
	set cache_dir_ [glob [$self get_option cacheDir]]
	set index_filename_ [file join $cache_dir_ index.db]
	if {! [catch {set f [open $index_filename_]}] } {
		while 1 {
			set line [gets $f]
			if [eof $f] {
				close $f
				break
			}
			set index_([lindex $line 0]) [lindex $line 1]
		}
	}

	# srmv2 related variables
	# session_ and source_ are arrays
	$self instvar port_ ttl_
	set ab [new AddressBlock $netspec]
	set addr [$ab addr]
	set port_ [$ab sport]
	set ttl_ [$ab ttl]
	delete $ab

	$self join_addr $addr

	# init mappings
	$self instvar addr_names_
	set addr_names_(*) $addr

	# as1 client
	$self instvar as_
	global params ssd
	set as_ [new AnnounceListenManager/AS/Client/CASA \
		$self [array get params] [array get ssd]]

	# start announcing data to servent
	$as_ start 	

	# build user interface
	if { [$self get_option recv] != "" } {
		$self instvar ui_
		set w .top$self
		frame $w
		set ui_ [new NewscastUI $w $self $as_]
		pack $w
		
	# configure webpoller
	} else {
		wm withdraw .
		$self instvar wp_ 
		foreach c $params(categories) {
			set wp_($c) [new WebPoller/NYTIndex $self $c]
		}
	}
}

NewscastApp private join_addr { addr } {
	$self instvar session_ source_ port_ ttl_
	
	set session_($addr) [srm_create_session $addr $port_ $port_ $ttl_]
	set source_($addr) [srm_create_source $session_($addr)]
	srm_callbacks $session_($addr) srm_recv srm_should_recover srm_read_adu \
		srm_source_update srm_recv_cid

	puts stderr "joining: $addr $session_($addr) $source_($addr)"				
}

NewscastApp private leave_addr { addr } {
	$self instvar session_ source_ 
	
	#srm_destroy_session $session_($addr)
	unset session_($addr)
	unset source_($addr)
	puts stderr "leaving: $addr"
}

NewscastApp private create_dir { path } {
	if { ![file isdirectory $path] } {
		set dir ""
		foreach split [file split $path] {
			set dir [file join $dir $split]
			if { ![file exists $dir] } {
				# this command will cause an error
				# if it is not possible to create the dir
				file mkdir $dir
			}
		}
	}
}

NewscastApp public recv_mappings { orphans mappings } {
	$self instvar addr_names_ 

	if { $mappings != "" } {
		unset addr_names_
		array set addr_names_ $mappings
		$self regroup
	}
}

NewscastApp public recv_bwalloc { bwalloc } {

}

#
# Join and leave multicast address depending on user interest
# and mapping information.
#
NewscastApp public regroup { } {
	$self instvar session_ source_ ui_ addr_names_

	# have data source subscribed to newly mapped addresses
	if { [$self get_option src] != "" } {
		set subscribed [array names session_]
		foreach ck [array names addr_names_] {
			if { ![info exists all] || [lsearch $all $addr_names_($ck)] == -1 } {
				lappend all $addr_names_($ck)
			}
		}
		set new [ldiff $ck $subscribed]
		foreach addr $new {
			$self join_addr $addr
		}

	# receivers
	} else {
		set subscribed [array names session_]
		foreach ck [array names addr_names_] {
			set addr $addr_names_($ck)
			set want [$ui_ is_interested $ck]
			set idx [lsearch $subscribed $addr]

			if { $want && $idx == -1 } {
				$self join_addr $addr
			} elseif { !$want && $idx != -1 } {
				$self leave_addr $addr
			}
		}
	}
}

#
# Given an address, find the current set of category,keywords pairs associated with it
#
NewscastApp private addr_2_names { addr } {
	$self instvar addr_names_

	foreach ck [array names addr_names_] {
		if { "$addr_names_($ck)" == "$addr" } {
			lappend all $ck
		}
	}
	if { ![info exists all] } {
		puts stderr "newscast: addr $addr not mapped"
		return ""
	} else {
		return $all
	}
}

#
# SRM related methods 
#

#
# Called by srm if someone needs to recover the specified data.
#
NewscastApp public read_data { source cid seqno } {
	$self instvar index_

	set url [$self cid_2_name $source $cid]
	
	if { $url == "" || ![info exists index_($url)] } {
		return ""
	} else {
		return $index_($url)
	}
}

#
# Given category and data, find the best address to send on
#
NewscastApp public choose_addr { category data } {
	$self instvar addr_names_

	# find addr that matches keyword in data
	foreach p [array names addr_names_] {
		set pair [split $p ,]
		set cat [lindex $pair 0]
		set kw [lindex $pair 1]
		if { "$cat" == "$category" && [$self keyword_match $kw $data] } {
			return [list $p $addr_names_($p)]
		}
	}
	
	# use category level addr
	set p "$category,*"
	if [info exists addr_names_($p)] {
		return [list $p $addr_names_($p)]

	# use root level addr
	} else {
		return [list * $addr_names_(*)]
	}
}

NewscastApp private keyword_match { kw data } {

	if { "$kw" == "*" } {
		return 1
	} else {
		# XXX need to fill in with grep 
		return 0
	}
}

#
# Called by webpoller when there is data to send.
#
NewscastApp public send_data { category url data } {
	$self instvar source_ index_ cid_names_ addr_names_

	puts stderr "SEND: $category $url $data"

	# find the mcast addr to send this data
	set pair [$self choose_addr $category $data]
	set ck [lindex $pair 0]	
	set addr [lindex $pair 1]

	# allocate a cid for this url if one does not exist
	set p [$self name_2_cid $ck]
	set source [lindex $p 0]
	set cid [lindex $p 1]
	if { $cid == "" || $source != $source_($addr)} {
		set cid [srm_calloc $source_($addr) 0 $ck]
		set cid_names_($source_($addr),$cid) $ck
	}

	set headline [lindex $data 0]
	set filename [lindex $data 1]
	set index_($url) $filename
	
	set f [open $index_($url)]
	fconfigure $f -translation binary
	set buffer ""
	while { ![eof $f] } {
		append buffer [read $f 4096]
	}
	close $f

	# send the data to the session 
	#set seqno [srm_send $source_($addr) $cid [list $headline $buffer]]
	set seqno [srm_send $source_($addr) $cid [list [list $headline $url] ""]]	
}


#
# Called when received data from the session.
#
NewscastApp public recv_data { source cid seqno data } {
	$self instvar index_ index_filename_ ui_ wp_ cache_dir_

	puts stderr "RECV: $source $cid $seqno $data"	

	# do a reverse mapping lookup to get url
	set ck [$self cid_2_name $source $cid]
	set cat [lindex $ck 0]
	set kw [lindex $ck 1]

	# store the data onto disk
	set headline [lindex [lindex $data 0] 0]
	set url [lindex [lindex $data 0] 1]
	set fn [lindex $data 1]

	set name cache[clock clicks]
	set index_($url) [file join $cache_dir_ $name[file extension $url]]

	file rename $fn $index_($url)

	# write the index file
	if [catch {set f [open $index_filename_ a]}] {
		set f [open $index_filename_ w 0644]
	}
	puts $f [list $url $index_($url)]
	close $f

	# display the data is there is interest in url
	$ui_ mkrow_hl "$headline" "$url"
}

#
# Default behavior is to recover always.
# Subclasses can change this reliability strategy.
#
NewscastApp public should_recover { source cid sseq eseq } {
	return 1
}


#
# Receive an sym to SRM mapping information.
#
NewscastApp public recv_cid { source cid parent_cid name } {
	$self instvar cid_names_
	set cid_names_($source,$cid) $name
}


#
# Given a cid, this method returns the application level
# name (sym) associated with it.
#
NewscastApp public cid_2_name { source cid } {
	$self instvar cid_names_
	
	if [info exists cid_names_($source,$cid)] {
		set url $cid_names_($source,$cid)
	} else {
		set url [srm_get_container_name $source $cid]
	}
	return $url
}

#
# Give a name (sym), this method returns the (source,cid) pair
# associated with it or an empty if it does not exist.
#
NewscastApp public name_2_cid { url } {
	$self instvar cid_names_

	foreach p [array names cid_names_] {
		if { "$cid_names_($p)" == "$url" } {
			return [split $p ,]
		}
	}
	return ""
}

#
# SRMv2 tcl callbacks
#

# XXX
set fn {/tmp/recv.dat}

proc srm_recv { src cid seqno data } {
	global newscast fn

	set f [open $fn w 0644]
	fconfigure $f -translation binary
	puts -nonewline $f [lindex $data 1]
	close $f
	
	$newscast recv_data $src $cid $seqno [list [lindex $data 0] $fn]
}

proc srm_read_adu { src cid seqno } {
	global newscast 
	
	#$newscast read_data $src $cid $seqno
}

proc srm_should_recover { src cid sseq eseq } {
	global newscast
	$newscast should_recover $src $cid $sseq $eseq
}


proc srm_recv_cid { src cid parent_cid name } {
	global newscast
	$newscast recv_cid $src $cid $parent_cid $name
}

proc srm_source_update { src info } {
	# do nothing now
}

