#
# Copyright (c) 1998 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#       This product includes software developed by the Computer Systems
#       Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# ------------
#
# Filename: ui-main.tcl
#   -- Author: Cynthia Romer <cromer@cs.berkeley.edu>
#
#  @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/mui/ui-main.tcl,v 1.20 1998/10/16 20:56:29 yatin Exp $
#


import TopLevelWindow UISrcList Observer Switcher \
		VicUI \
		TkWindow HelpWindow InfoWindow RtpStatWindow ErrorWindow \
		ScubaInfoWindow \
		ControlMenu MuiHelpWindow VideoBoxManager AudioPanel \
		UserWindow ActiveSource ActiveSourceManager Icons Configuration VisualFrame 

#XX
import ErrorWindow

#
# An integrated interface to the MASH tools.  Currently interfaces to
# many of the capabilities provided by mb, vic, and vat.
#
# <br> Old defaults:
#	<br> minwidth 650
#	<br> minheight 450
Class MuiUI -superclass VicUI -configuration {
	meterEnable true
}
	
#
# Build the user-interface.
# Pack it in the frame, <i>w</i>, which the caller is responsible for instantiating and packing.
# <br> <i>video_args</i> should be a list composed of a VideoAgent and a VideoPipeline
# <br> <i>audio_args</i> should be a list composed of a AudioAgent
# <br> <i>mb_args</i> should be a list composed of an mbManager and an mbSender
#
MuiUI public init { w localChannel globalChannel video_args audio_args mb_args exitCmd {vspec {}} } {
	$self instvar have_video_ have_audio_ have_mb_
	set have_video_ [expr {[llength $video_args] > 0}]
	set have_audio_ [expr {[llength $audio_args] > 0}]
	set have_mb_ [expr {[llength $mb_args] > 0}]
	
	$self instvar audioAgent_ mbMgr_ mbSndr_
	set videoAgent [lindex $video_args 0]
	set vpipe [lindex $video_args 1]
	set audioAgent_ [lindex $audio_args 0]
	set mbMgr_ [lindex $mb_args 0]
	set mbSndr_ [lindex $mb_args 1]
	
	#vbx temporary hack while MuiUI is a subclass of VicUI; used to be $self next $w  
	$self next $w $localChannel $globalChannel $videoAgent $vpipe "$exitCmd" $vspec $have_video_
}

# accessor function
MuiUI public mbui {} {
	$self instvar mbui_
	return $mbui_
}

#
# Layout the user-interface by creating frames within widget <i>w</i> for various UI components and groups of components.
#
MuiUI public layout_gui {w} {
	$self instvar have_video_ have_audio_ have_mb_
	$self instvar vframe_ 
	
	set gui $w.vframe
	set vframe_ [new VisualFrame $gui $self]

	frame $gui.row1
	frame $gui.row2
	frame $gui.row3
	frame $gui.row2.left
	frame $gui.row2.right
	frame $gui.row2.left.lower
	frame $gui.row2.left.upper

	$self instvar exitCmd_
	if $have_video_ {
		$self instvar videoAgent_ controlMenu_ vpipe_ asm_ localChannel_
		if $have_mb_ {
			set asm_ [new ActiveSourceManager $self $gui $videoAgent_ horizontal $localChannel_]	
		} else {
			set asm_ [new ActiveSourceManager $self $gui $videoAgent_ vertical $localChannel_]	
		}
		foreach i { 1 2 3 4 } {
			bind . <Key-$i> "$asm_ redecorate $i"
		}
		
		set controlMenu_ [new ControlMenu $self $videoAgent_ $vpipe_ $vframe_ $asm_ [new UISrcListWindow $gui $videoAgent_]]
		#XXX a bit dangerous to have this in the GUI
		bind . <t> "$controlMenu_ build_window ; $controlMenu_ invoke_transmit"

		$self build_menubar $gui.menuPanel [new MuiHelpWindow .help] "$exitCmd_" $controlMenu_ 
	} else {
		$self build_menubar $gui.menuPanel [new MuiHelpWindow .help] "$exitCmd_"
	}

	if $have_video_ {
		lappend panels [frame $gui.awarenessPanel]
		$self make_awareness_panel $gui.awarenessPanel
		
		lappend panels [frame $gui.videoPanel -height 576 -width 652]
		$self make_video_panel $gui.videoPanel
	}

	if $have_mb_ {
		lappend panels [frame $gui.mbPanel] 
		$self make_mb_panel $gui.mbPanel
	}
	
	if $have_audio_ {
		lappend panels [frame $gui.audioPanel]
		$self make_audio_panel $gui.audioPanel
	}
		
	# This should be replaced with tina's cues.
	# These buttons this creates didn't invoke any commands; they were just for show.
	lappend panels [frame $gui.cuePanel]
	# $self make_cues_panel $gui.cuePanel

	# Should somehow try to incorporate rover's vcr panel.
	lappend panels [frame $gui.vcrPanel]
	# $self make_vcr_panel $gui.vcrPanel

	foreach panel $panels {
		$panel config -relief ridge -borderwidth 2
	}

	# pack panels
	pack $gui.menuPanel -expand 1 -fill both -in $gui.row1
	if $have_mb_ {
		pack_if_exists $gui.awarenessPanel -expand 0 -fill both -in $gui.row2.left.upper
	} else {
		pack_if_exists $gui.awarenessPanel -expand 0 -fill both -side left -in $gui.row2.left.lower
	}
	pack_if_exists $gui.videoPanel -expand 1 -fill both -side right -in $gui.row2.left.lower 
	pack_if_exists $gui.mbPanel -expand 1 -fill both -side left -in $gui.row2.left.lower 
	pack_if_exists $gui.vcrPanel -expand 1 -fill both -in $gui.row3
	pack_if_exists $gui.cuePanel -expand 0 -fill both -in $gui.row2.right
	pack_if_exists $gui.audioPanel -expand 1 -fill both -in $gui.row2.right

	# pack rows
	pack $gui.row2.left.upper -expand 0 -fill x -side top
	pack $gui.row2.left.lower -expand 1 -fill both -side top

	pack $gui.row2.right -fill both -side right
	pack $gui.row2.left -expand 1 -fill both -side left

	pack $gui.row1 -fill x -side top
	pack $gui.row2 -expand 1 -fill both -side top
	pack $gui.row3 -fill x -side bottom -before $gui.row2

	# pack it all
	pack $gui -expand 1 -fill both

#	pack $label_ -anchor c -expand 1 -side left -fill both
}

#
# This is the same as the "pack" command, except no error is
# generated if "bad window path name"
#
proc pack_if_exists { window_path_names args } {
	foreach window_path_name $window_path_names {
		if [winfo exists $window_path_name] {
			eval {pack $window_path_name} $args
		}
	}
}

#
MuiUI public make_awareness_panel { w } {
	$self instvar asm_
	$asm_ init_grid $w.grid
	pack $w.grid 
	#	scrollbar $w.scrollbar -orient horizontal -width 12
	#	pack $w.scrollbar -side bottom		
}

#
MuiUI public make_video_panel { w } {
	#vbx        frame $w.vb1 -relief ridge -borderwidth 2
	#vbx        frame $w.vb2 -relief ridge -borderwidth 2
	
	$self instvar vbm_ asm_
	set vbm_ [new VideoBoxManager $asm_ 2 $w {-size medium -border false -nametag true -updateSpeed normal}]
	
	foreach vb [$vbm_ get_videoboxes] {
		pack $vb -expand 1 -fill both 
	}
	
	#	label $w.label -text "Videos go here"
	#	pack $w.label -anchor c -expand 1 -side left -fill both
}

#
MuiUI public make_mb_panel { w } {
	$self instvar exitCmd_ mbui_ mbMgr_ mbSndr_ 
	#	label $w.label -text "Mediaboard goes here"
	#	pack $w.label -anchor c -expand 1 -side left -fill both
	set mbui_ [new MBUI $w $mbMgr_ $mbSndr_ [$self get_option showUI] "$exitCmd_"]
}

#
MuiUI public make_audio_panel { w } {
	$self instvar audioAgent_
	set panel_ [new AudioPanel $w $audioAgent_]
	global outputScale inputScale
	$outputScale set [$audioAgent_ get_output_gain]
	$inputScale set [$audioAgent_ get_input_gain]
	set meterEnable [$self yesno meterEnable]
	$panel_ enable_meters $meterEnable
	#	label $w.label -text "audio panel goes here"
	#	pack $w.label -anchor c -expand 1 -side left -fill both
}

#
MuiUI public make_cues_panel { w } {
	button $w.earbutton -text Help -relief raised \
			-image Icons(ear) \
			-borderwidth 2 -height 24 -highlightthickness 1
	pack $w.earbutton -side left -expand 1 -fill x
	button $w.handbutton -text Help -relief raised \
			-image Icons(hand) \
			-borderwidth 2 -height 24 -highlightthickness 1
	pack $w.handbutton -side left -expand 1 -fill x
	#	label $w.label -text "cues"
	#	pack $w.label -anchor c -expand 1 -side left -fill both
}

#
MuiUI public make_vcr_panel { w } {
	label $w.label -text "VCR panel goes here"
	pack $w.label -anchor c -expand 1 -side left -fill both
}

#
# Create the main menubar for the MuiApplication.
# Takes a frame, <i>w</i>, and create and pack within it a row of buttons.
#
MuiUI public build_menubar { w helpWindow exitCmd {controlWindow {}} } {
	global title
	frame $w -relief ridge -borderwidth 2

        menubutton $w.join -text "Join Session" -underline 0 -menu $w.join.menu 
        menu $w.join.menu
        $w.join.menu add cascade -label "Bookmarks" -menu $w.join.bookmarks
        $w.join.menu add command -label "Browse Directory"
        $w.join.menu add command -label "Browse Archive"
        menu $w.join.bookmarks
        $w.join.bookmarks add command -label "UCB CSCW"
        $w.join.bookmarks add command -label "UCB CSCW Playback"
        $w.join.bookmarks add command -label "MASH/Daedulus Group"
        $w.join.bookmarks add command -label "Berkeley Multimedia and Graphics Seminar"

        menubutton $w.switch -text "Switch Session" -underline 0 -menu $w.switch.menu
        menu $w.switch.menu
        $w.switch.menu add command -label "UCB CSCW"
        $w.switch.menu add command -label "UCB CSCW Playback"

        menubutton $w.exit -text "Exit Session" -underline 0 -menu $w.exit.menu
        menu $w.exit.menu
        $w.exit.menu add command -label "Exit Current Session"
        $w.exit.menu add command -label "Exit All Sessions"

	if { $controlWindow != {} } {
		menubutton $w.prefs -text "Control" -underline 0 -menu $w.prefs.menu
		menu $w.prefs.menu    
		$w.prefs.menu add command -label "Video" -command "$controlWindow toggle"
#		button $w.menu -text Menu -relief raised \
#				-font [$self get_option smallfont] -highlightthickness 1 \
#				-command "$controlWindow toggle"
	}
	
        menubutton $w.profiles -text "Member Profiles" -underline 0 -menu $w.profiles.menu
        menu $w.profiles.menu
        $w.profiles.menu add command -label "Edit Mine"
        $w.profiles.menu add command -label "View All"

        menubutton $w.restart -text "Restart" -underline 0 -menu $w.restart.menu
        menu $w.restart.menu 
        $w.restart.menu add command -label "All"
        $w.restart.menu add command -label "MediaBoard"
        $w.restart.menu add command -label "Audio"
        $w.restart.menu add command -label "Video"
        $w.restart.menu add command -label "Player"
        $w.restart.menu add command -label "Recorder"

        menubutton $w.help -text "Help" -underline 0 -menu $w.help.menu 
        menu $w.help.menu
        $w.help.menu add command -label "Help" -command "$helpWindow toggle"
#        $w.help.menu add command -label "Email Cindy"
#	button $w.help -text Help -relief raised \
#		-font [$self get_option smallfont] -highlightthickness 1 \
#		-command "$helpWindow toggle"

#	label $w.title -text "MUI v[version]" -font [$self get_option smallfont] \
#		-relief flat -justify left
	button $w.quit -text Quit -underline 0 -command $exitCmd 

#	pack $w.title -side left -fill both -expand 1
#	pack $w.menu $w.help $w.quit -side left -padx 1 -pady 1

        pack $w.quit $w.help -side right -padx 5 -pady 1	
#	pack $w.restart -side right -padx 5 -pady 1
#	pack $w.join $w.switch $w.exit -side left -padx 5 -pady 1
	pack_if_exists $w.prefs -side left -padx 5 -pady 1
#	pack $w.profiles -side left -padx 5 -pady 1
}

#
# Update the source description by updating the element of the global
# arrays src_info(), src_nickname(), and src_name() indexed by the
# specified <i>src</i>.  If the src_name() is changed, then this is
# reflected in the UserWindows in which this src appears.
#
MuiUI public trigger_sdes src {
	$self instvar asm_ vbm_
	
	global src_info src_nickname src_name src_cname
	#
	# Figure out best presentation from available information.
	#
	set name [$src sdes name]
	set cname [$src sdes cname]
	set addr [$src addr]

	regsub "Recorded Source\:" "$name" "" name_tail
	regsub "@.*" "$cname" "" cname_head
        set name $name_tail
        set cname $cname_head

	if { $name == "" } {
		if { $cname == "" } {
			set src_nickname($src) $addr
			set info $addr/[$src format_name]

		} else {
			set src_nickname($src) $cname
			set info "$addr/[$src format_name]"
		}
	} elseif [cname_redundant $name $cname] {
		set src_nickname($src) $name
		set info $addr/[$src format_name]
	} else {
		set src_nickname($src) $name
		set info $cname/[$src format_name]
	}
	set msg [$src sdes note]
	if { $msg != "" } {
		set info $msg
	}
	set src_info($src) $info

	$self instvar have_video_
	# only call change_name when name really changes
	if { ![info exists src_name($src)] || "$src_name($src)" != "$name" } {
		set src_name($src) $name
		if $have_video_ {
			$asm_ change_name $src
		}
	}

	if { ![info exists src_cname($src)] || "$src_cname($src)" != "$cname" } {
		set src_cname($src) $cname
		if $have_video_ {
			$vbm_ maybe_switch_in $src
		}
	}
}

#vbxmgr
#saving for ui-activesourcemgr.tcl
MuiUI public maybe_switch_in src {
	$self instvar vbm_
	$vbm_ maybe_switch_in $src
}

#
# A toggle-able dismissable toplevel window for displaying a bulleted list of tips to help the mui user.
#
Class MuiHelpWindow -superclass HelpWindow

#
# Instantiate, but do not yet display or iconify, a dismissable toplevel
# window using the provided widgetpath, <i>w</i>.  Within <i>w</i>,
# create a bulleted list of tips to help the mui user.  (This method is
# called when the user toggles this window for the first time.)
#
MuiHelpWindow public build w {

	$self create-window $w "Help" {

"Transmit video by clicking on the ``Transmit'' button \
in the ``Menu'' window.  You need video capture hardware to do this."

"Incoming video streams appear in the main vic window.  \
If you see the message ``Waiting for video...'', then no one is transmitting \
video to the conference address you're running on.  Otherwise, you'll \
see a thumbnail sized image and accompanying information for each source. \
Click on the thumbnail to open a larger viewing window.  You can tile the \
thumbnails in multiple columns using the ``Tile'' menu in the ``Menu'' window."
"Clicking on the ``mute'' button for a given source will \
turn off decoding.  It is usually a good idea to do \
this for your own, looped-back transmission."

"The transmission rate is controlled with the bit-rate \
and frame-rate sliders in the ``Transmission'' panel of the ``Menu'' window.  \
The more restrictive setting limits the transmission rate."

"The video windows need not be fixed to a given source. \
The ``Mode...'' menu attached to a viewing window allows you to specify \
voice-switched and/or timer-switched modes.   In timer-switched mode, the \
window automatically cycles through (unmuted) sources, while in \
oice-switched mode, the window switches to whomever is talking \
(using cues from vat).  You can have more than one voice-switched window, \
which results in a simple LRU allocation of the windows to most recent \
speakers.  See the man page for more details."

"If the user interface looks peculiar, you might \
have X resources that conflict with tk.  A common problem is \
defining ``*background'' and/or ``*foreground''."

"Bugs and suggestions to mash-developers@mash.cs.berkeley.edu.  Thanks."
	}
}

